<!---
Copyright (C) 2024, UChicago Argonne, LLC
Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
in the top-level directory.
--->

## ---- plqstat.py ----
### show me the list of jobs like qstat
python plqstat.py

### sow me the list of jobs with score and comment.  FIXME: make comment more generate.  in header?
python plqstat.py --score --comment


## ---- plnodes.py ----
### give me simular output to pbsnodes
python plnodes.py

### output the nodes with the additional resource ncpus
python plnodes.py -r ncpus

### give a list of vnodes, one per line.  #FIXME: add delim other than \n
python plnodes.py --header vnode --bare


## ---- pllog.py ----
### enhance the log and dump it to a file.
python pllog.py pipe --pipein /var/spool/pbs/server_logs/20231004 --pipeout /tmp/out.txt

### enhance the log and dump it to a stdout
python pllog.py pipe --pipein /var/spool/pbs/server_logs/20231004

### enhance the log and dump it to a stdout.  gzip is only supported via --pipein
python pllog.py pipe --pipein /var/spool/pbs/server_logs/20230925.gz

### process a log via pipe.
cat /var/spool/pbs/server_logs/20231004 | python pllog.py pipe


## ---- plquery.py ----

### this is where data will be dumped and read from using -l or a specific prefix
export PLPATH=/data

### --filt applies a logical_and of the selectors.
### --filt-or applies a logical_or of the selectors.
### --agg, --groupby are quite complex.
### --bare should be used with a single header value to create a list.  You can change the delimiter using --delim.

### first decide if you want to query the system every time or use cached data.
### if you want to use latest, set the variable to True and dump the data, it will go into PLPATH
export PLLATEST=True
python plquery.py dump

### dump data with pbsnodes and qstat
python plquery.py dump --dpn --dqs

### from this point on, if PLLATEST is True, it will use the data in PLPATH.

### what are the jobs on the system and vnode if running.
python plquery.py -l job

### what is the state of all jobs and count of nodes if running
python plquery.py -l --groupby jobid --groupby job_state --agg vnode=count job

### what size job can I run?
python plquery.py -l --agg isavailable=sum avail

python plquery.py -l --groupby jobid --groupby job_state --agg vnode=count job

### which nodes are free?
python plquery.py -l --filt isavailable=1 avail

### give me a list of nodes that are free in a comma separated list.
python plquery.py -l --filt isavailable=1 --header vnode --bare avail

### special command
### pbsnodes -a -F json | jq -r '.nodes|.[]|select(.resources_available.broken == "True")|[.resources_available.host, .state, .comment] | join(",")'
python plquery.py -l --header vnode,resources_available.broken --filt resources_available.broken=False vnode
python plquery.py -l --header vnode --filt resources_available.broken=False --header vnode --bare vnode
python plquery.py -l --header vnode,resources_available.broken --filt resources_available.broken=True vnode

### all queues with all jobs and their states.
python plquery.py -l queue-jobs-agg

### all queues with only jobs in the running state
python plquery.py -l -f job_state=running queue-jobs-agg

### how many possible nodes are in each queue and how many are available.
python plquery.py -l queue-avail

### what size job can I run for queue x?
python plquery.py -l --filt queue=somequeue1 queue-avail

### How many nodes are free in all queues?
python plquery.py -l --groupby isavailable --agg isavailable=count avail
### How many nodes are free in all queues and some weird sum of the available nodes, not useful.
python plquery.py -l --groupby isavailable --agg isavailable=count,sum avail

### How many nodes are under a reservation, but the node state is not correct?
python plquery.py -l -vvv --filt node_state=free --filt isavailable=0 avail
python plquery.py -l -vvv --filt node_state=free --filt isavailable=0 --agg vnode=count avail

### how many nodes are available to a reservation
python plquery.py -l --filt queue=somequeue1 queue-avail

### which nodes are free for my reservation queue?
python plquery.py -l --filt queue=somequeue1 avail

### how many nodes are "down"
python plquery.py -l --agg isdown=count,sum --filt isdown=1 avail --filt-in-or node_state_lst=down,offline,state-unknown,Stale

### how many nodes are "down" or isavailable=0,1.  Not useful but shows what could be done.
python plquery.py -l --agg isdown=count,sum --filt isdown=1 avail --filt-in-or node_state_lst=down,offline,state-unknown,Stale --filt-in-or isavailable=0,1

### give me the list of vnodes with the validation resource = True
python plquery.py -l --filt resources_available.validation=True --header vnode,comment vnode

### give me all the down nodes and comments.
python plquery.py -l --header last_state_change_time,last_used_time,vnode,node_state,comment vnode|egrep "(down|offline)"|grep -v "free"|sort -k 1 -t " "

### give me the list of nodes in a queue with a default_chunk
python plquery.py -l --filt queue=validation avail
python plquery.py -l --filt queue=validation --filt isavailable=1 avail

### give me the list of nodes that are available in the queue
python plquery.py -l --header vnode --bare --filt queue=validation --filt isavailable=1 avail
python plquery.py -l --header vnode --bare --bare-delim ☃ --filt queue=validation --filt isavailable=1 avail

### give me the vnode information for the 3 vnodes
python plquery.py -l --filt queue=workq --filt-or vnode=pdw-c02 --filt-or vnode=pdw-c01 --filt-or vnode=pdw-c03 avail

### give me the vnode information for the 3 vnodes, but only if it's available.
python plquery.py -l --filt queue=workq --filt isavailable=1 --filt-or vnode=pdw-c02 --filt-or vnode=pdw-c01 --filt-or vnode=pdw-c03 avail

### give me all the jobs that have not progresses in walltime, stuck
python plquery.py -l job-special --option stuck

### give me all the jobs that have not progresses in walltime, stuck, just the jobid
python plquery.py -l --header jobid --bare job-special --option stuck

## ---- plctrl.py ----
### update the mom config with the debug level of 4095 along with 3 --usecp's, restarts pbs #FIXME: restart correctly.
python plctrl.py mom-config 4095 --usecp '*:/fs/ /fs/' --usecp '*:/home/ /home/' --usecp '*:/dev/null /dev/null'

### create many moms, 10624 to be exact.  Probably requires sudo.  --pstart is the start port and --pend is the port end.
plctrl.py create-moms --mom pdw-c01 --name pdw-c01 --pstart 15005 --pend 25629

### delete those 10624 moms
plctrl.py delete-moms --mom pdw-c01 --name pdw-c01 --pstart 15005 --pend 25629
