# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import os
import pytest
import traceback
from pprint import pformat
from click.testing import CliRunner
from PBS_Utils.scripts.plquery import cli

try:
    import pbs_ifl
except ImportError:
    pbs_ifl = None
LOADDATA_DIR = os.path.join(os.path.dirname(__file__), "data/pldata")


def get_data_prefix(prefix_lst=None):
    if prefix_lst is None:
        prefix_lst = [
            ("-l",),  # latest data
            ("--data-prefix", "20231003T150529"),  # 1 job in queue, pdw-c03 offline, 3 free, one reservation confirmed.
            ("--data-prefix", "20231003T150639"),  # no jobs, no reservations, all 4 nodes free.
            (
                "--data-prefix",
                "20231003T151028",
            ),  # 1 running job, one stuck job, pdw-c03 offline, 2 free, one resv confirmed, one resv running."
        ]
    call_lst = [
        ("job",),
        (
            "--groupby",
            "jobid",
            "--groupby",
            "job_state",
            "--agg",
            "vnode=count",
            "job",
        ),
        (
            "--agg",
            "isavailable=sum",
            "avail",
        ),
        (
            "--filt",
            "isavailable=1",
            "avail",
        ),
        (
            "--filt",
            "isavailable=1",
            "--header",
            "vnode",
            "--bare",
            "avail",
        ),
        (
            "--header",
            "vnode,resources_available.ncpus",
            "--filt",
            "resources_available.ncpus=16",
            "vnode",
        ),
        (
            "--header",
            "vnode",
            "--filt",
            "resources_available.ncpus=16",
            "--header",
            "vnode",
            "--bare",
            "vnode",
        ),
        (
            "--header",
            "vnode,resources_available.ncpus",
            "--filt",
            "resources_available.ncpus=0",
            "vnode",
        ),
        ("queue-jobs-agg",),
        (
            "-f",
            "job_state=running",
            "queue-jobs-agg",
        ),
        ("queue-avail",),
        (
            "--filt",
            "queue=default",
            "queue-avail",
        ),
        (
            "--groupby",
            "isavailable",
            "--agg",
            "isavailable=count",
            "avail",
        ),
        (
            "--groupby",
            "isavailable",
            "--agg",
            "isavailable=count,sum",
            "avail",
        ),
        (
            "--filt",
            "node_state=free",
            "--filt",
            "isavailable=0",
            "avail",
        ),
        (
            "--filt",
            "node_state=free",
            "--filt",
            "isavailable=0",
            "--agg",
            "vnode=count",
            "avail",
        ),
        (
            "--filt",
            "queue=somequeue",
            "queue-avail",
        ),
        (
            "--filt",
            "queue=somequeue",
            "avail",
        ),
        (
            "--agg",
            "isdown=count,sum",
            "--filt",
            "isdown=1",
            "avail",
            "--filt-in-or",
            "node_state_lst=down,offline,state-unknown,Stale",
            "--filt-in-or",
            "isavailable=0,1",
        ),
        ("--header", "vnode", "--bare", "--bare-delim", "☃", "--filt", "queue=validation", "--filt", "isavailable=1", "avail"),
        (
            "--filt",
            "queue=workq",
            "--filt-or",
            "vnode=pdw-c02",
            "--filt-or",
            "vnode=pdw-c01",
            "--filt-or",
            "vnode=pdw-c03",
            "avail",
        ),
        (
            "--filt",
            "queue=workq",
            "--filt",
            "isavailable=1",
            "--filt-or",
            "vnode=pdw-c02",
            "--filt-or",
            "vnode=pdw-c01",
            "--filt-or",
            "vnode=pdw-c03",
            "avail",
        ),
        (
            "job-special",
            "--option",
            "stuck",
        ),
        (
            "--header",
            "jobid",
            "--bare",
            "job-special",
            "--option",
            "stuck",
        ),
    ]
    test_lst = []
    for prefix in prefix_lst:
        for call in call_lst:
            test_lst.append([*prefix, *call])
    return test_lst


@pytest.mark.skipif(pbs_ifl is None, reason="could not import pbs_ifl")
def test_dump_ifl_00(tmp_path):
    runner = CliRunner()
    result = runner.invoke(cli, ["dump", "--dpn", "--dqs"], env={"PLPATH": str(tmp_path)})
    assert result.exit_code == 0, pformat(traceback.format_exception(*result.exc_info))


@pytest.mark.parametrize("option", get_data_prefix())
def test_cli_00(option):
    """test the use of job with various datasets."""
    runner = CliRunner()
    result = runner.invoke(cli, option, env={"PLPATH": LOADDATA_DIR})
    assert result.exit_code == 0, pformat(traceback.format_exception(*result.exc_info))


@pytest.mark.parametrize(
    "option",
    get_data_prefix(
        prefix_lst=[
            (),
        ]
    ),
)
@pytest.mark.skipif(pbs_ifl is None, reason="could not import pbs_ifl")
def test_job_ifl_00(option):
    runner = CliRunner()
    result = runner.invoke(cli, option, env={"PLPATH": LOADDATA_DIR})
    assert result.exit_code == 0, pformat(traceback.format_exception(*result.exc_info))
